from __future__ import annotations

from pathlib import Path
from tempfile import TemporaryDirectory as _TemporaryDirectory
from tempfile import gettempdir as _gettempdir
from typing import Any

from typing_extensions import override

from utilities.pathlib import PathLike


class TemporaryDirectory(_TemporaryDirectory[Any]):
    """Sub-class of TemporaryDirectory whose name attribute is a Path."""

    def __init__(
        self,
        *,
        suffix: str | None = None,
        prefix: str | None = None,
        dir: PathLike | None = None,  # noqa: A002
    ) -> None:
        super().__init__(suffix=suffix, prefix=prefix, dir=dir)
        self.name = Path(self.name)

    @override
    def __enter__(self) -> Path:
        return super().__enter__()


def gettempdir() -> Path:
    """Get the name of the directory used for temporary files."""
    return Path(_gettempdir())


TEMP_DIR = gettempdir()



from __future__ import annotations

from collections.abc import Sequence

from click import command, echo
from click.testing import CliRunner
from hypothesis import given
from hypothesis.strategies import integers, none
from pytest import mark, param

from utilities.click.luigi import (
    local_scheduler_option_default_central,
    local_scheduler_option_default_local,
    workers_option,
)


class TestLocalSchedulerOption:
    @mark.parametrize(
        ("args", "expected"),
        [param([], True), param(["-ls"], True), param(["-nls"], False)],
    )
    def test_default_local(
        self, *, args: Sequence[str], expected: bool
    ) -> None:
        @command()
        @local_scheduler_option_default_local
        def cli(*, local_scheduler: bool) -> None:
            echo(f"local_scheduler = {local_scheduler}")

        result = CliRunner().invoke(cli, args)
        assert result.exit_code == 0
        assert result.stdout == f"local_scheduler = {expected}\n"

    @mark.parametrize(
        ("args", "expected"),
        [param([], False), param(["-ls"], True), param(["-nls"], False)],
    )
    def test_default_central(
        self, *, args: Sequence[str], expected: bool
    ) -> None:
        @command()
        @local_scheduler_option_default_central
        def cli(*, local_scheduler: bool) -> None:
            echo(f"local_scheduler = {local_scheduler}")

        result = CliRunner().invoke(cli, args)
        assert result.exit_code == 0
        assert result.stdout == f"local_scheduler = {expected}\n"


class TestWorkersOption:
    @given(workers=integers() | none())
    def test_main(self, workers: int | None) -> None:
        @command()
        @workers_option
        def cli(*, workers: int | None) -> None:
            echo(f"workers = {workers}")

        args = [] if workers is None else ["--workers", str(workers)]
        result = CliRunner().invoke(cli, args)
        assert result.exit_code == 0
        assert result.stdout == f"workers = {workers}\n"
